#include "grbc/state.h"
#define SOL_ALL_SAFETIES_ON 1

#include "grbc/helpers.h"
#include "grbc/spec.h"
#include <sol/sol.hpp>

int main() {
  log_msg("opening lua libraries...");

  auto &lua = GState::get().lua;

  lua.open_libraries(sol::lib::base, sol::lib::coroutine, sol::lib::string,
                     sol::lib::io, sol::lib::package, sol::lib::math);

  log_msg("setting up runtime...");

  // TargetInfo
  lua.new_usertype<TargetInfo>("TargetInfo", "name", &TargetInfo::name);

  // PackageConfig
  lua.new_usertype<PackageConfig>(
      "PackageConfig", "name", &PackageConfig::name, "libraries",
      &PackageConfig::libraries, "include_dirs", &PackageConfig::include_dirs,
      "compile_flags", &PackageConfig::compile_flags, "linker_flags",
      &PackageConfig::linker_flags);

  // GlobalConfig
  lua.new_usertype<GlobalConfig>(
      "GlobalConfig", "engine_version", &GlobalConfig::engine_version,
      "architecture", &GlobalConfig::architecture, "target",
      &GlobalConfig::target, "target_config", &GlobalConfig::target_config);

  // Platform
  lua.new_usertype<Platform>("Platform", "name", &Platform::name,
                             "cxx_compiler", &Platform::cxx_compiler,
                             "cc_compiler", &Platform::cc_compiler,
                             "platform_type", &Platform::platform_type, "is_64bit", &Platform::is_64bit);

  // PlatformType
  lua.new_enum<PlatformType>("PlatformType", {{"Win32", PlatformType_Win32},
                                              {"Linux", PlatformType_Linux}});

  // LanguageType
  lua.new_enum<LanguageType>("LanguageType", {
    { "Cpp", LanguageType_CPP },
    { "C", LanguageType_C }
  });

  // ExecutableConfig
  lua.new_usertype<ExecutableConfig>(
      "ExecutableConfig", sol::constructors<ExecutableConfig(sol::table)>(),
      "name", &ExecutableConfig::name, "files", &ExecutableConfig::files,
      "requirments", &ExecutableConfig::requirements, "compile_flags",
      &ExecutableConfig::compile_flags, "linker_flags",
      &ExecutableConfig::linker_flags, "include_dirs",
      &ExecutableConfig::include_dirs, "language_type", &ExecutableConfig::language_type);

  // Package
  lua.new_usertype<Package>("Package", sol::constructors<Package(sol::table)>(), "name", &Package::name, "compiler_flags",
                            &Package::compiler_flags, "linker_flags",
                            &Package::linker_flags);

  lua.set("grbc_want_version", grbc_want_version);
  lua.set("grbc_exception", grbc_exception);
  lua.set("grbc_get_config", grbc_get_config);
  lua.set("grbc_file", grbc_file);
  lua.set("grbc_executable", grbc_executable);
  lua.set("grbc_load_platform", grbc_load_platform);
  lua.set("grbc_set_platform", grbc_set_platform);
  lua.set("grbc_find_compiler", grbc_find_compiler);
  lua.set("grbc_build", grbc_build);
  lua.set("grbc_replace_string", grbc_replace_string);
  lua.set("grbc_object_file", grbc_object_file);
  lua.set("grbc_include_dirs_to_clflags", grbc_include_dirs_to_cflags);
  lua.set("grbc_log", grbc_log);
  lua.set("grbc_is_32bit", grbc_is_32bit);
  lua.set("grbc_is_64bit", grbc_is_64bit);
  lua.set("grbc_is_linux", grbc_is_linux);
  lua.set("grbc_is_win32", grbc_is_win32);
  lua.set("grbc_get_platform", grbc_get_platform);

  // Load generators
  
  GState::get().generators.push_back({
    .name = "ninja",
    .func = ninja_generator
  });

  log_msg("loading HConfig...");

  lua.script_file("HConfig");
}