#include "grbc/helpers.h"
#include "grbc/spec.h"
#include <cstdlib>
#include <ctime>
#include <filesystem>

void grbc_exception(const std::string &exception_string) {
  printf("--- GRBC EXCEPTION: ---\n");
  printf("Message: %s\n", exception_string.c_str());
  printf("Timestamp: %ld", time(nullptr));

  exit(EXIT_FAILURE);
}

void grbc_want_version(const std::string &version) {
  if (grbc_get_config().engine_version != version) {
    grbc_exception("Expected engine_version " + version + ", got " +
                   grbc_get_config().engine_version);
  }
}

std::string grbc_object_file(const std::string &file_path) {
    std::filesystem::path path(file_path);
    path.replace_extension(".o");

    std::string path_str = path.generic_string();

    // Strip out stuff
    std::string no_spaces = grbc_replace_string(path_str, ' ', '_');

    return no_spaces;
}

std::string grbc_replace_string(const std::string &string, char substring,
                                char replacement) {

  std::string res;

  for (char cChar : string) {
    if (cChar == substring) {
      res += replacement;
    } else {
      res += cChar;
    }
  }

  return res;
}

GlobalConfig grbc_get_config() {
  static GlobalConfig cfg{};

  return cfg;
}

std::string grbc_include_dirs_to_cflags(const std::vector<std::string> &include_dirs) {
  std::string result;
  
  for (auto &include_dir : include_dirs) {
    result += "-I" + include_dir + " ";
  }

  result.pop_back(); // Remove trailing space

  return result;
}

void grbc_log(const std::string &message) {
  log_msg(message.c_str());
}